﻿#include "StdAfx.h"
#include "rhinoSdkPlugInDeclare.h"
#include "TestPlugIn.h"
#include "Resource.h"

// The plug-in object must be constructed before any plug-in classes derived
// from CRhinoCommand. The #pragma init_seg(lib) ensures that this happens.
#pragma warning(push)
#pragma warning(disable : 4073)
#pragma init_seg(lib)
#pragma warning(pop)

// Rhino plug-in declaration
RHINO_PLUG_IN_DECLARE
RHINO_PLUG_IN_NAME(L"Test");
RHINO_PLUG_IN_ID(L"259F53C3-89F1-4E2A-A7AF-0B7E9F88CD78");
RHINO_PLUG_IN_VERSION(__DATE__ "  " __TIME__)
RHINO_PLUG_IN_DESCRIPTION(L"Test plug-in for Rhinoceros®");
RHINO_PLUG_IN_ICON_RESOURCE_ID(IDI_ICON);
RHINO_PLUG_IN_DEVELOPER_ORGANIZATION(L"Robert McNeel & Associates");
RHINO_PLUG_IN_DEVELOPER_ADDRESS(L"3670 Woodland Park Avenue North\r\nSeattle WA 98103");
RHINO_PLUG_IN_DEVELOPER_COUNTRY(L"United States");
RHINO_PLUG_IN_DEVELOPER_PHONE(L"206-545-6877");
RHINO_PLUG_IN_DEVELOPER_FAX(L"206-545-7321");
RHINO_PLUG_IN_DEVELOPER_EMAIL(L"devsupport@mcneel.com");
RHINO_PLUG_IN_DEVELOPER_WEBSITE(L"http://www.rhino3d.com");
RHINO_PLUG_IN_UPDATE_URL(L"https://github.com/mcneel/rhino-developer-samples");

// The one and only CTestPlugIn object
static class CTestPlugIn thePlugIn;

// The one and only CFileEventWatcher object
static class CFileEventWatcher theFileEventWatcher;

// The one and only CIdleEventWatcher object
static class CIdleEventWatcher theIdleEventWatcher;

/////////////////////////////////////////////////////////////////////////////
// CTestPlugIn definition

CTestPlugIn& TestPlugIn()
{
	// Return a reference to the one and only CTestPlugIn object
	return thePlugIn;
}

CTestPlugIn::CTestPlugIn()
{
	m_plugin_version = RhinoPlugInVersion();
}

/////////////////////////////////////////////////////////////////////////////
// Required overrides

const wchar_t* CTestPlugIn::PlugInName() const
{
	return RhinoPlugInName();
}

const wchar_t* CTestPlugIn::PlugInVersion() const
{
	return m_plugin_version;
}

GUID CTestPlugIn::PlugInID() const
{
	// {259F53C3-89F1-4E2A-A7AF-0B7E9F88CD78}
	return ON_UuidFromString(RhinoPlugInId());
}

/////////////////////////////////////////////////////////////////////////////
// Additional overrides

BOOL CTestPlugIn::OnLoadPlugIn()
{
  // Register and enabled file event watcher
  theFileEventWatcher.Register();
  theFileEventWatcher.Enable(true);

  // Register and disable idle event watcher
  theIdleEventWatcher.Register();
  theIdleEventWatcher.Enable(false);

	return TRUE;
}

void CTestPlugIn::OnUnloadPlugIn()
{
}

CRhinoPlugIn::plugin_load_time CTestPlugIn::PlugInLoadTime()
{
  return CRhinoPlugIn::load_plugin_at_startup;
}

/////////////////////////////////////////////////////////////////////////////
// CFileEventWatcher definition

void CFileEventWatcher::OnNewDocument(CRhinoDoc& doc)
{
  UNREFERENCED_PARAMETER(doc);

  // Enable idle event watcher
  theIdleEventWatcher.Enable(true);
}

void CFileEventWatcher::OnEndOpenDocument(CRhinoDoc& doc, const wchar_t* filename, BOOL bMerge, BOOL bReference)
{
  UNREFERENCED_PARAMETER(doc);
  UNREFERENCED_PARAMETER(filename);
  UNREFERENCED_PARAMETER(bMerge);
  UNREFERENCED_PARAMETER(bReference);

  // Enable idle event watcher
  theIdleEventWatcher.Enable(true);
}

/////////////////////////////////////////////////////////////////////////////
// CIdleEventWatcher definition

CIdleEventWatcher::CIdleEventWatcher()
  : CRhinoIsIdle(thePlugIn.PlugInID())
{
}

void CIdleEventWatcher::Notify(const class CRhinoIsIdle::CParameters& params)
{
  // Disable idle event watcher
  theIdleEventWatcher.Enable(false);

  // Get the active docment
  CRhinoDoc* doc = RhinoApp().ActiveDoc();
  if (nullptr == doc)
    return;

  // Get all of the model views
  ON_SimpleArray<CRhinoView*> view_list;
  const int view_count = doc->GetViewList(view_list, true, false);
  if (0 == view_count)
    return;

  // Find a model view that is in a perspective projection
  CRhinoView* active_view = nullptr;
  for (int i = 0; i < view_count; i++)
  {
    CRhinoView* view = view_list[i];
    if (nullptr != view)
    {
      if (view->ActiveViewport().VP().IsPerspectiveProjection())
      {
        active_view = view;
        break;
      }
    }
  }

  if (nullptr == active_view)
    return;

  // Set as the active view
  RhinoApp().SetActiveView(active_view);

  // Set the display mode to shaded
  active_view->ActiveViewport().SetDisplayMode(CRhinoDisplayAttrsMgr::ShadedModeId());

  // Maximize view if needed
  if (!active_view->IsMaximized())
    active_view->MaximizeRestoreView();

  // Redraw for good measure
  active_view->Redraw();
}
